﻿using Microsoft.AspNetCore.Cors;
using Microsoft.AspNetCore.Mvc;
using Newtonsoft.Json.Linq;
using DbCycleVDP;

namespace APICycleVDP.Controller
{
    [EnableCors("All")]
    public class CyclesController : ControllerBase
    {
        [HttpPost]
        public ActionResult<string> CurrCycles()
        {
            var result = new JObject
            {
                ["currTime"] = DateTime.Now
            };

            var tasks = new List<Task<TableCycle>>();
            //var v = new DB.WorkDB();
            for (var i = 1; i <= 48; i++)
                tasks.Add(WorkDB.GetCycle(i));

            var arr = new JArray();
            for (var i = 0; i < tasks.Count; i++)
            {
                var tmp = tasks[i].GetAwaiter().GetResult();
                if (tmp != null)
                {
                    var t = new JObject();
                    t["vdp"] = tmp.NumVdp;
                    t["cycle"] = tmp.NumCycle;
                    t["factStart"] = tmp.FactStart;
                    t["factEnd"] = tmp.FactEnd;
                    t["thinkEnd"] = tmp.ThinkEnd;
                    arr.Add(t);
                }
            }
            result["data"] = arr;

            return Content(result.ToString(), "application/json");
        }
    }


    public static class WorkDB
    {
        async public static Task<TableCycle> GetCycle(int vdp)
        {
            var result = new TableCycle();
            await Task.Run(() =>
            {
                try
                {
                    using var db = new DbFurnace();
                    var tmp = (from u in db.Cycles
                               where
                                    u.NumVdp == vdp
                               orderby u.FactStart descending
                               select u).FirstOrDefault();
                    result = tmp;
                }
                catch (Exception e)
                {
                    Console.WriteLine(e.Message);
                }
            });
            return result;
        }
    }
}
